///@file sample2.cpp - how to insert and select data in SQLite3's database.
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sqlite3.h"

#include "subfunc.cpp"	// IP and FILE I/O functions.

/** print result of SQL select. this is called by sqlite3_exec() */
int select_callback(void*, int argc, char** argv, char** columnNames)
{
	// argv[] has a result set of SQL "SELECT ip1,ip2,ip3,ip4,net,count(*) FROM ....;"
	printf("IP:[%s.%s.%s.%s], NET:[%s], %s:%s\n",
		argv[0],	// ip1
		argv[1],	// ip2
		argv[2],	// ip3
		argv[3],	// ip4
		argv[4],	// net
		columnNames[5], argv[5]	// count(*)
		);
	return 0;
}

/** program main.
 * usage: sample2 [dbfile [mailbox]]
 * this program works:
 *  - scan received header which contain IP-adderss number from mailbox
 *  - insert them into dbfile
 *  - select top 10 IPs from dbfile
 *  - dbfile default: sample.db
 *  - mailbox default: stdin
 */
int main(int argc, char** argv)
{
	sqlite3* db;
	int err;
	char* db_name = (argc > 1) ? argv[1] : "sample.db";
	char* sql;

	// open input
	FILE* fin = (argc > 2) ? fopen(argv[2], "rb") : stdin; // use "rb" to input beyond Ctrl-Z.
	if (!fin) {
		perror(argv[2]);
		exit(1);
	}

	// open database
	err = sqlite3_open(db_name, &db);

	if (err != SQLITE_OK) {
		// show error and exit.
		fprintf(stderr, "sqlite3_open: %s\n", sqlite3_errmsg(db));
		exit(1);
	}

	// create "mail" table
	err = sqlite3_exec(db,
		"CREATE TABLE mail("
		  "ip1 INTEGER, ip2 INTEGER, ip3 INTEGER, ip4 INTEGER, net TEXT,"
		  "is_private TINYINT, header TEXT);",
		NULL, 0, 0);

	if (err != SQLITE_OK)
		fprintf(stderr, "sqlite3_exec(CREATE TABLE): %s\n", sqlite3_errmsg(db));

	// begin transaction
	sqlite3_exec(db, "BEGIN;", NULL, NULL, NULL);

	char buf[1024*8];
	while (fgets(buf, sizeof(buf), fin)) {
		// parse a received header-line from input
		int c;
		unsigned ip[4];
		if (strncmp(buf, "Received: from ", 15) != 0 || !scan_ip(buf+15, ip))
			continue;

		while ((c = fpeek(fin)) == ' ' || c == '\t')
			fcatgets(buf, sizeof(buf), fin);	// append lines until header end.

		const char* net = net_address(ip);
		bool is_private = is_private_ip(ip);

		// insert IP and header-line into "mail" table
		sql = sqlite3_mprintf(
			"INSERT INTO mail(ip1,ip2,ip3,ip4,net,is_private,header) VALUES(%u,%u,%u,%u,'%s',%d,'%q');",
			ip[0], ip[1], ip[2], ip[3], net, is_private, buf);

		err = sqlite3_exec(db, sql, NULL, NULL, NULL);

		if (err != SQLITE_OK)
			fprintf(stderr, "sqlite3_exec(INSERT INTO mail): %s\n", sqlite3_errmsg(db));

		sqlite3_free(sql);
	}//.endwhile

	// commit transaction
	sqlite3_exec(db, "COMMIT;", NULL, NULL, NULL);

	// select top 10 ip from "mail" table.
	puts("** Top 10 IPs **");
	err = sqlite3_exec(db,
		"SELECT ip1,ip2,ip3,ip4,net,count(*) FROM mail"
		"  GROUP BY ip1,ip2,ip3,ip4 HAVING is_private=0"
		"  ORDER BY count(*) DESC LIMIT 10;",
		select_callback, NULL, NULL);

	if (err != SQLITE_OK)
		fprintf(stderr, "sqlite3_exec(SELECT): %s\n", sqlite3_errmsg(db));

	// close database
	sqlite3_close(db);

	// close input
	fclose(fin);

	return EXIT_SUCCESS;
}
// sample2.cpp - end.
