///@file sample3.cpp - how to prepare and step in SQLite3's database.
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sqlite3.h"

#include "subfunc.cpp"	// IP and FILE I/O functions.

/** print result of SQL select using by sqlite3_column_xxx() */
void print_result(sqlite3_stmt* select_sql)
{
	// print result set of SQL "SELECT ip1,ip2,ip3,ip4,net,count(*) FROM ....;"
	printf("IP:[%d.%d.%d.%d], NET:[%s], %s:%s\n",
		sqlite3_column_int(select_sql, 0),	// ip1
		sqlite3_column_int(select_sql, 1),	// ip2
		sqlite3_column_int(select_sql, 2),	// ip3
		sqlite3_column_int(select_sql, 3),	// ip4
		sqlite3_column_text(select_sql, 4),	// net
		sqlite3_column_name(select_sql, 5),	// count(*)
		sqlite3_column_text(select_sql, 5)	// count(*)
	);
}

/** program main.
 * usage: sample3 [dbfile [mailbox]]
 * this program works:
 *  - scan received header which contain IP-adderss number from mailbox
 *  - insert them into dbfile
 *  - select top 10 IPs from dbfile
 *  - dbfile default: sample.db
 *  - mailbox default: stdin
 */
int main(int argc, char** argv)
{
	sqlite3* db;
	int err;
	char* db_name = (argc > 1) ? argv[1] : "sample.db";
	char* sql;

	// open input
	FILE* fin = (argc > 2) ? fopen(argv[2], "rb") : stdin; // use "rb" to input beyond Ctrl-Z.
	if (!fin) {
		perror(argv[2]);
		exit(1);
	}

	// open database
	err = sqlite3_open(db_name, &db);

	if (err != SQLITE_OK) {
		// show error and exit.
		fprintf(stderr, "sqlite3_open: %s\n", sqlite3_errmsg(db));
		exit(1);
	}

	// create "mail" table
	err = sqlite3_exec(db,
		"CREATE TABLE mail("
		  "ip1 INTEGER, ip2 INTEGER, ip3 INTEGER, ip4 INTEGER, net TEXT,"
		  "is_private TINYINT, header TEXT);",
		NULL, 0, 0);

	if (err != SQLITE_OK)
		fprintf(stderr, "sqlite3_exec(CREATE TABLE): %s\n", sqlite3_errmsg(db));

	// make prepared SQL for mail table
	sqlite3_stmt* insert_sql;
	sqlite3_stmt* select_sql;
	sql = "INSERT INTO mail(ip1,ip2,ip3,ip4,net,is_private,header) VALUES(?,?,?,?,?,?,?);";
	sqlite3_prepare(db, sql, strlen(sql), &insert_sql, NULL);
	const int bind_net = 5;
	const int bind_is_private = 6;
	const int bind_header = 7;

	sql = "SELECT ip1,ip2,ip3,ip4,net,count(*) FROM mail"
		"  GROUP BY ip1,ip2,ip3,ip4 HAVING is_private=0"
		"  ORDER BY count(*) DESC LIMIT 10;";
	sqlite3_prepare(db, sql, strlen(sql), &select_sql, NULL);

	// begin transaction
	sqlite3_exec(db, "BEGIN;", NULL, NULL, NULL);

	char buf[1024*8];
	while (fgets(buf, sizeof(buf), fin)) {
		// parse a received header-line from input
		int c;
		unsigned ip[4];
		if (strncmp(buf, "Received: from ", 15) != 0 || !scan_ip(buf+15, ip))
			continue;

		while ((c = fpeek(fin)) == ' ' || c == '\t')
			fcatgets(buf, sizeof(buf), fin);	// append lines until header end.

		const char* net = net_address(ip);
		bool is_private = is_private_ip(ip);

		// insert IP and header-line into "mail" table
		sqlite3_reset(insert_sql);
		sqlite3_bind_int(insert_sql, 1, ip[0]);
		sqlite3_bind_int(insert_sql, 2, ip[1]);
		sqlite3_bind_int(insert_sql, 3, ip[2]);
		sqlite3_bind_int(insert_sql, 4, ip[3]);
		sqlite3_bind_int(insert_sql, bind_is_private, is_private);
		sqlite3_bind_text(insert_sql, bind_net,    net, strlen(net), SQLITE_STATIC);
		sqlite3_bind_text(insert_sql, bind_header, buf, strlen(buf), SQLITE_STATIC);
		err = sqlite3_step(insert_sql);

		if (err != SQLITE_DONE)
			fprintf(stderr, "sqlite3_step(INSERT INTO mail): %s\n", sqlite3_errmsg(db));
	}//.endwhile

	// commit transaction
	sqlite3_exec(db, "COMMIT;", NULL, NULL, NULL);

	// select top 10 ip from "mail" table.
	puts("** Top 10 IPs **");
	while ((err = sqlite3_step(select_sql)) == SQLITE_ROW)
		print_result(select_sql);

	if (err != SQLITE_DONE)
		fprintf(stderr, "sqlite3_step(SELECT): %s\n", sqlite3_errmsg(db));

	// discard prepared SQL
	sqlite3_finalize(insert_sql);
	sqlite3_finalize(select_sql);

	// close database
	sqlite3_close(db);

	// close input
	fclose(fin);

	return EXIT_SUCCESS;
}
// sample3.cpp - end.
